import os
os.environ['TF_CPP_MIN_LOG_LEVEL'] = '3' # Disable logging messages from tensorflow

import warnings
import torch
import tensorflow as tf
import numpy as np
import pandas as pd
from deepcoil.utils.seqvec import SeqVec
from deepcoil.utils.encoders import SeqVecMemEncoder
from deepcoil.utils.generators import SeqChunker
from deepcoil.utils import corr_seq
from tensorflow.keras.models import model_from_json
from tensorflow.keras.utils import get_file
import tensorflow.keras.backend as K
from zipfile import ZipFile


class DeepCoil:

    def __init__(self, use_gpu=True, n_cpu=-1):
        """
        API for DeepCoil - a predictor of coiled-coil domains in protein sequences
        :param n_cpu: (int) - number of CPUs for inference without GPU. n_cpu=-1 will use all available resources.
        :param use_gpu: (bool) - flag for enabling or disabling GPU
        """

        self.use_gpu = use_gpu
        if n_cpu != -1:
            torch.set_num_threads(n_cpu)
        self._path = os.path.dirname(os.path.abspath(__file__))

        # Set the weights and model dir
        self._weights_prefix = f'{self._path}/weights/seq'
        self._model_loc = f'{self._path}/models/seq.json'
        self._n_weights = 5

        # Handle GPU config
        if self.use_gpu:
            available_devices = tf.config.list_physical_devices('GPU')
            if len(available_devices) == 0:
                raise SystemError('No GPU device is available!')
            elif len(available_devices) > 1:
                warnings.warn(f'Multiple GPU devices are available, will use device: {available_devices[0]}')
            # Set the memory limit on tf/keras prediction since SeqVec encoding will happen also on the same GPU
            tf.config.experimental.set_virtual_device_configuration(available_devices[0], [
                tf.config.experimental.VirtualDeviceConfiguration(memory_limit=1024)])
        else:
            # Disable GPU's if some will get automatically detected
            tf.config.set_visible_devices([], 'GPU')

        # Build DeepCoil model
        self._seqvec = self._setup_seqvec()
        self.model = self._setup_model()

    def _setup_seqvec(self):
        """
        Load SeqVec model to either GPU or CPU, if weights are not cached - download them from the mirror.
        :return: instance of deepcoil.utils.seqvec.SeqVec
        """
        seqvec_dir = f'{self._path}/weights/seqvec'
        seqvec_conf_fn = f'{seqvec_dir}/uniref50_v2/options.json'
        seqvec_weights_fn = f'{seqvec_dir}/uniref50_v2/weights.hdf5'
        if not (os.path.isfile(seqvec_conf_fn) and os.path.isfile(seqvec_weights_fn)):
            print('SeqVec weights are not available, downloading from the remote source (this\'ll happen only once)...')
            urls = ['https://rostlab.org/~deepppi/seqvec.zip', 'https://lbs.cent.uw.edu.pl/static/files/seqvec.zip']
            for url in urls:
                try:
                    seqvec_zip_fn = get_file(f'{self._path}/weights/seqvec.zip', url)
                    archive = ZipFile(seqvec_zip_fn)
                    archive.extract('uniref50_v2/options.json', seqvec_dir)
                    archive.extract('uniref50_v2/weights.hdf5', seqvec_dir)
                    break
                except:
                    print(f'Could not download SeqVec weights from url: {url}')
        if self.use_gpu:
            return SeqVec(model_dir=f'{seqvec_dir}/uniref50_v2', cuda_device=0, tokens_per_batch=8000)
        else:
            return SeqVec(model_dir=f'{seqvec_dir}/uniref50_v2', cuda_device=-1, tokens_per_batch=8000)

    def _setup_model(self):
        """
        Setup appropriate DeepCoil model based on the json file definition
        :return: loaded keras model
        """
        with open(self._model_loc) as f:
            conf_json = f.read()
        return model_from_json(conf_json)

    def _process_input(self, data):
        """
        Validate and process the input eventually filtering/correcting problematic sequences
        :param data: input data dict
        :return: pd.DataFrame with sequences and IDs
        """
        if not isinstance(data, dict):
            raise ValueError('Input data must be a dictionary with ids and sequences as keys and values!')
        valid_seqs = [isinstance(seq, str) for seq in data.values()]
        if not all(valid_seqs):
            raise ValueError('Input data must be a dictionary with ids and sequences as keys and values!')
        if len(data) == 0:
            raise ValueError('Empty dictionary was passed as an input!')
        non_std_seqs = [sequence != corr_seq(sequence) for sequence in data.values()]
        too_short_seqs = [len(sequence) < 20 for sequence in data.values()]
        if all(too_short_seqs):
            raise ValueError('Input sequence(s) are below 20aa length!')

        # Convert input dict to pd.DataFrame for easy handling
        data = pd.DataFrame.from_dict(data, orient='index', columns=['sequence'])

        if any(non_std_seqs):
            data['sequence'] = data['sequence'].apply(lambda x: corr_seq(x))
            warnings.warn('Non-standard residues detected in input data were corrected to X token.', UserWarning)

        if any(too_short_seqs):
            data = data[data['sequence'].str.len() >= 20]
            warnings.warn('Filtered out input sequences shorter than 20 residues.', UserWarning)

        return data

    def predict(self, data):
        """
        Evaluate sequence(s) with DeepCoil
        :param data: (dict) - dictionary containing ids (dict keys) sequences (dict values) to evaluate with DeepCoil
        :return:
        """

        data = self._process_input(data)

        # Initialize dict with preds per fold values which latter will be averaged
        cc_preds_per_fold = {key: [] for key in data.index.tolist()}
        hept_preds_per_fold = {key: [] for key in data.index.tolist()}
        # Encode SeqVec (#TODO: handle large inputs which may cause OOM errors)
        embeddings = self._seqvec.encode(data, to_file=False)

        # Setup generator that'll be evaluated
        seqvec_enc = SeqVecMemEncoder(embeddings, pad_length=500)
        gen = SeqChunker(data, batch_size=64, W_size=500, shuffle=False,
                         data_encoders=[seqvec_enc], data_cols=['sequence'])

        if self.use_gpu:
            # If GPU is used free mem used by SeqVec
            torch.cuda.empty_cache()

        # Predict with each of N predictors, depad predictions and average out for final output
        for i in range(1, self._n_weights+1):
            self.model.load_weights(f'{self._weights_prefix}_{i}.h5')

            # Raw predictions
            cc_preds, hept_preds = self.model.predict(gen, steps=len(gen), verbose=0, workers=1)

            # Remove padding and join predictions if in more than one chunk
            cc_preds_depadded = {key: np.concatenate([cc_preds[ix][ind[0]:ind[1]] for ix, ind in zip(*value)])
                                 for key, value in gen.indices.items()}
            hept_preds_depadded = {key: np.concatenate([hept_preds[ix][ind[0]:ind[1]] for ix, ind in zip(*value)])
                                   for key, value in gen.indices.items()}

            # Aggregate predictions for each fold
            for key, pred in cc_preds_depadded.items():
                cc_preds_per_fold[key].append(pred)
            for key, pred in hept_preds_depadded.items():
                hept_preds_per_fold[key].append(pred)

        K.clear_session()

        # Average predictions between 5 predictors from CV training
        cc_preds_avg = {key: np.average(value, axis=0).flatten() for key, value in cc_preds_per_fold.items()}
        hept_preds_avg = {key: np.average(value, axis=0) for key, value in hept_preds_per_fold.items()}
        # Final results
        results = {key: {'cc': probs, 'hept': hept_preds_avg[key]} for key, probs in cc_preds_avg.items()}
        return results



import pandas as pd
import numpy as np
import os
from allennlp.commands.elmo import ElmoEmbedder


class SeqVec:

    def __init__(self, model_dir, cuda_device=-1, tokens_per_batch=16000):
        """
        Wrapper for efficient embedding of protein sequences with SeqVec (Heinzinger et al., 2019)
        :param model_dir: Directory storing SeqVec files (weights.hdf5 and options.json)
        :param cuda_device: Index of the CUDA device to use when encoding (-1 if CPU)
        :param tokens_per_batch: Number of tokens (amino acids per encoded sequence batch) - depends on available RAM
        """
        weights = model_dir + '/' + 'weights.hdf5'
        options = model_dir + '/' + 'options.json'
        self.seqvec = ElmoEmbedder(options, weights, cuda_device=cuda_device)
        self.tokens_per_batch = tokens_per_batch

    def encode(self, data, to_file=True, out_path=None, sum_axis=True, cut_out=False):
        """
        Encodes sequences stored in 'data' DataFrame
        :param data: pandas DataFrame storing sequences ('sequence' column) and optionally 'beg' and 'end' indices
        to cut out the embeddings
        :param to_file: If true save embedding for further use in 'out_path'
        :param out_path: Directory to store embeddings if to_file is True. Filenames match the indexes of the 'data'.
        :param sum_axis: Specifies whether first axis of the embedding will be summed up.
        This will results in Nx1024 embedding for a protein sequence of length N.
        :param cut_out: Optionally cut the embedding with the 'beg' and 'end' indices. Useful when calculating the
        embedding for whole sequence and cutting out only part of it. If True data must contain 'beg' and 'end' columns.
        :return results: if 'to_file' is false returns dictionary with data indexes as keys and embedding as values.
        """
        # Validate input DataFrame
        if not isinstance(data, pd.DataFrame):
            raise TypeError('Data must be a pandas DataFrame!')
        if 'sequence' not in data.columns:
            raise ValueError('DataFrame must contain sequence column!')
        if cut_out and 'beg' not in data.columns and 'end' not in data.columns:
            raise ValueError('DataFrame must contain beg and end columns to if cut_out is True!')
        if to_file and not os.path.isdir(out_path):
            raise OSError('Output directory does not exist!')

        # Process input DataFrame
        tmp_df = data.copy()
        tmp_df['seq_len'] = tmp_df['sequence'].apply(len) # Calculate length of each sequence in DataFrame
        tmp_df = tmp_df.sort_values(by='seq_len') # Sort sequences by length
        tmp_df['cum_seq_len'] = tmp_df['seq_len'].cumsum() # Calculate cumulative sequence lengths to split into batches
        tmp_df['batch'] = tmp_df['cum_seq_len'] // self.tokens_per_batch
        # Encode sequences in batches to speed up the process. Each batch contain at most 'tokens_per_batch' aa's.
        results = {}
        for batch in tmp_df['batch'].unique():
            df = tmp_df[tmp_df['batch'] == batch]
            sequences = df['sequence'].tolist()
            if cut_out:
                beg_indices = df['beg'].tolist()
                end_indices = df['end'].tolist()
            embs = self.seqvec.embed_sentences(sequences)
            # Sum first axis if specified
            if sum_axis:
                embs = [emb.sum(axis=0) for emb in embs]
            # Cut out sequence chunks if specified
            if cut_out:
                embs = [emb[beg:end] for emb, beg, end in zip(embs, beg_indices, end_indices)]
            # Save results
            for emb, _id in zip(embs,  df.index.values):
                if to_file:
                    np.save('{}/{}.npy'.format(out_path, _id), emb)
                else:
                    results[_id] = emb
        if not to_file:
            return results




import numpy as np


class SeqVecEncoder:

    def __init__(self, seqvec_dir=None, pad_length=100):
        """
        Encoder for SeqVec to use with the generators in the mltoolbox.keras.generators. Uses precalculated raw numpy
        arrays stored in the 'seqvec_dir'.
        :param seqvec_dir: Directory storing encoded seqvec numpy arrays.
        """
        self.seqvec_dir = seqvec_dir
        self.pad_length = pad_length

    def _encode_seqvec(self, _id, beg, end, pad_left=0):
        fn = '{}/{}.npy'.format(self.seqvec_dir, _id)
        matrix = np.load(fn)[beg:end, :]
        pad_matrix = np.zeros((self.pad_length, 1024))
        pad_matrix[pad_left:matrix.shape[0] + pad_left, 0:matrix.shape[1]] = matrix
        return pad_matrix

    def encode_batch(self, batch_df, i):
        X = np.asarray(
            [self._encode_seqvec(value['id'], value['beg'], value['end'], pad_left=value['pad_left']) for key, value in
             batch_df.iterrows()])
        return X


class SeqVecMemEncoder:

    def __init__(self, seqvec_dict=None, pad_length=100):
        """
        Encoder for SeqVec to use with the generators in the mltoolbox.keras.generators. Uses precalculated raw numpy
        arrays stored in the 'seqvec_dir'.
        :param seqvec_dir: Directory storing encoded seqvec numpy arrays.
        """
        self.seqvec_dict = seqvec_dict
        self.pad_length = pad_length

    def _encode_seqvec(self, _id, beg, end, pad_left=0):
        matrix = self.seqvec_dict[_id][beg:end, :]
        pad_matrix = np.zeros((self.pad_length, 1024))
        pad_matrix[pad_left:matrix.shape[0] + pad_left, 0:matrix.shape[1]] = matrix
        return pad_matrix

    def encode_batch(self, batch_df, i):
        X = np.asarray(
            [self._encode_seqvec(value['id'], value['beg'], value['end'], pad_left=value['pad_left']) for key, value in
             batch_df.iterrows()])
        return X



import numpy as np
import matplotlib.pyplot as plt
import seaborn as sns
from itertools import groupby
from Bio import SeqIO
from scipy.signal import find_peaks


# Adapted from https://stackoverflow.com/questions/44293407/how-can-i-check-whether-a-given-file-is-fasta
def is_fasta(filename):
    """
    Checks whether given file is in fasta format
    :param filename: name of the file to check
    :return: True/False denoting whether file is in fasta fortmat
    """
    with open(filename, 'r') as handle:
        fasta = SeqIO.parse(handle, 'fasta')
        return any(fasta)


def corr_seq(seq):
    """
    Corrects sequence by mapping non-std residues to 'X'
    :param seq: input sequence
    :return: corrected sequence with non-std residues changed to 'X'
    """
    letters = set(list('ACDEFGHIKLMNPQRSTVWYX'))
    seq = ''.join([aa if aa in letters else 'X' for aa in seq])
    return seq


def sharpen_preds(probs):
    """
    Sharpens raw probabilities returned by DeepCoil to more human-readable format
    :param probs: raw probabilities
    :return: sharpened probabilities
    """
    sharp_probs = np.zeros(len(probs))
    peaks = find_peaks(probs.flatten(), width=7, rel_height=0.6)
    for i in range(0, len(peaks[0])):

        beg = int(peaks[1]['left_ips'][i])
        end = int(peaks[1]['right_ips'][i])
        prob = max(probs[beg:end])

        for j in range(beg, end + 1):
            if prob >= 0.1:
                sharp_probs[j] = prob

    sharp_probs = sharp_probs.flatten()
    above_threshold = sharp_probs > 0
    for k, g in groupby(enumerate(above_threshold), key=lambda x: x[1]):
        if k:
            g = list(g)
            beg, end = g[0][0], g[-1][0]
            sharp_probs[beg:end] = max(sharp_probs[beg:end])

    return sharp_probs


def plot_preds(results, beg=0, end=-1, out_file=None, dpi=300):
    """
    Helper function for plotting DeepCoil results
    :param results: results for given entry returned by DeepCoil
    :param beg: (optional) beginning aa of the range to use for plotting (useful for long sequences to see only subset of results)
    :param end: (optional) end aa of the range to use for plotting (useful for long sequences to see only subset of results)
    :param out_file: (optional) if specified results will also be dumped to file
    :param dpi: (optional) DPI of the resulting image
    :return:
    """

    sharp_probs = sharpen_preds(results['cc'])[beg:end]
    probs, hept = results['cc'][beg:end], results['hept'][beg:end, :]

    fig, (ax2, ax1) = plt.subplots(2, gridspec_kw={'height_ratios': [1, 9]}, figsize=(9, 5))

    # Plot probs and sharpened probs
    ax1.plot(probs, linewidth=1, c='gray', linestyle='dashed')
    ax1.plot(sharp_probs, linewidth=2, c='black')

    # Set axis limits
    ax1.set_ylim(0.01, 1)
    ax1.set_xlim(0, len(probs))

    # Show grid
    ax1.grid(linestyle="dashed", color='gray', linewidth=0.5)

    # Find and set appropriate spacing of xticks given the sequence length
    spacings = [10, 25] + list(range(50, 1000, 50))
    spacing = spacings[np.argmin([abs(10 - len([i for i in range(0, len(probs), spacing)])) for spacing in spacings])]
    ticks = [i for i in range(0, len(probs), spacing)]
    labels = [i + beg for i in range(0, len(probs), spacing)]
    ax1.set_xticks(ticks)
    ax1.set_xticklabels(labels)
    ax1.set_yticks([i / 10 for i in range(1, 10, 1)])
    ax1.xaxis.set_ticks_position('none')
    ax1.set_xlabel('Position in Sequence')
    ax1.set_ylabel('Coiled Coil Probablity')

    # Parse a, d heptad annotations and plot
    a, d = [], []
    for i, pr in enumerate(hept):
        if pr[1] > 0.2 or pr[2] > 0.2:
            if pr[1] > pr[2]:
                a.append(pr[1])
                d.append(0)
            else:
                d.append(-pr[2])
                a.append(0)
        else:
            a.append(0)
            d.append(0)
    kk = np.vstack((a, d))
    sns.heatmap(np.asarray(kk), cmap='bwr', vmin=-1, vmax=1, cbar=None, ax=ax2)

    # Hide bottom panel of the subplot
    for name, spine in ax2.spines.items():
        if name != 'bottom':
            spine.set_visible(True)

    # Show the ticks corresponding to the bottom panel
    ax2.xaxis.grid(linestyle="dashed", color='gray', linewidth=0.5)
    ax2.set_xticks([i for i in range(0, len(probs), 50)])
    ax2.set_yticks([0.5, 1.5])
    ax2.set_yticklabels(['\'a\' core pos.', '\'d\' core pos.'], rotation=0)
    ax2.set_xticks(ticks)
    ax2.xaxis.set_ticks_position('none')

    plt.subplots_adjust(hspace=0)
    if out_file:
        plt.savefig(out_file, dpi=dpi)
        plt.close()


import random
import pandas as pd
import numpy as np
from tensorflow import keras

class SeqChunker(keras.utils.Sequence):
    def __init__(self, data, W_size=64, batch_size=64, shuffle=True, neg_frac=1,
                 data_encoders=None, label_encoders=None, data_cols=None, label_cols=None):
        """
        Generates sequence chunks of fixed sized window with defined overlap between windows
        :param data: instance of the pandas DataFrame containing the dataset
        :param W_size: size of the sequence windows
        :param batch_size: size of the batches returned by the generator
        :param shuffle: True/False - shuffle entries between each epoch
        :param neg_frac: fraction of negatives used at each epoch. 'global_label' columns must be defined in the 'data'
        if neg_frac < 1
        :param data_encoders: list of data encoders
        :param label_encoders: list of label encoders
        :param data_cols: columns in the 'data' DataFrame that'll be used by the data_encoders
        (length of data_cols list must be equal to the length of data_encoders list)
        :param label_cols: columns in the 'data' DataFrame that'll be used by the label_encoders
        (length of label_cols list must be equal to the length of label_encoders list)
        """
        self.data = data.copy()
        self.batch_size = batch_size
        self.data_encoders = data_encoders
        self.data_cols = data_cols
        self.label_encoders = label_encoders
        self.label_cols = label_cols

        self.W_size = W_size
        self.shuffle = shuffle
        self.neg_frac = neg_frac

        # Validate input
        self._validate_input()
        # Select only neccesary data
        cols = []
        if self.neg_frac < 1:
            cols += ['global_label']
        if self.label_cols is not None:
            cols += self.label_cols
        cols += self.data_cols
        self.data = self.data[cols]
        self._assign_batches()

    def _validate_input(self):
        if not isinstance(self.data, pd.DataFrame): # Input data must be pd.DataFrame
            raise TypeError('Data must be a pandas DataFrame!')
        if not isinstance(self.data_encoders, list): # Encoders must be passed as the list
            raise ValueError('Data encoder(s) must be specified as list!')
        if len(self.data_encoders) != len(self.data_cols): # Number of data encoders must be equal to number of columns
            raise ValueError('Number of data encoders must be equal to number of data columns in the data df')
        if self.label_encoders is not None:
            if not isinstance(self.label_encoders, list): # Encoders must be passed as the list
                raise ValueError('Label encoder(s) must be specified as list!')
            if len(self.label_encoders) != len(self.label_cols): # Number of label encoders must be equal to number of columns
                raise ValueError('Number of label encoders must be equal to number of label columns in the data df')
        if self.neg_frac < 1 and 'global_label' not in self.data.columns: # 'global_label' must be passed for downsampling negatives
            raise ValueError('Dataframe must contain global_label column for neg class downsampling')
        if self.label_encoders is not None: # Check for the label format (single or seq2seq)
            self._single_label = [all(len(str(label)) == 1 for label in self.data[label_col].values) for label_col in
                                  self.label_cols]
        self._single_data = [all(len(str(inp)) == 1 for inp in self.data[data_col].values) for data_col in
                             self.data_cols]

    def _assign_batches(self):
        if self.neg_frac < 1:
            tmp_dict = self.data.loc[
                set(self.data[self.data['global_label'] == 0].sample(frac=self.neg_frac).index) | set(
                    self.data[self.data['global_label'] == 1].index)].to_dict(orient='index')
        else:
            tmp_dict = self.data.to_dict(orient='index')

        if self.label_encoders is not None:
            windowed_data = {indice[0]: indice[1:] for name, value in tmp_dict.items() for indice in
                             self._split_seq(name, data=[value[col] for col in self.data_cols],
                                             labels=[value[col] for col in self.label_cols])}
        else:
            windowed_data = {indice[0]: indice[1:] for name, value in tmp_dict.items() for indice in
                             self._split_seq(name, data=[value[col] for col in self.data_cols])}

        self.windowed_data = pd.DataFrame.from_dict(windowed_data, orient='index',
                                                    columns=['id', 'data', 'label', 'beg', 'end', 'pad_left'])

        miss_idxs = set(tmp_dict.keys()) - set(self.windowed_data.id.values)
        for idx in miss_idxs:
            tmp_dict.pop(idx)

        if self.shuffle:
            self.windowed_data = self.windowed_data.sample(frac=1)
        indices = self.windowed_data.groupby(by='id').indices
        pads_left = self.windowed_data.groupby(by='id').agg({'pad_left': list})['pad_left'].to_dict()
        self.windowed_data['len'] = self.windowed_data['end'] - self.windowed_data['beg']
        lens = self.windowed_data.groupby(by='id').agg({'len': list})['len'].to_dict()
        self.indices = {key: (
            list(indices[key]), [(pad_left, pad_left + len_) for pad_left, len_ in zip(pads_left[key], lens[key])]) for key in tmp_dict.keys()}
        self.windowed_data['batch'] = np.arange(len(self.windowed_data)) // self.batch_size

    def _split_seq(self, name, data=None, labels=None):
        seq = data[0]
        n_windows = 1 + len(seq) // self.W_size
        pad_left = (n_windows * self.W_size - len(seq)) // 2
        pad_right = (n_windows * self.W_size - len(seq)) // 2
        if (n_windows * self.W_size - len(seq)) % 2 != 0:
            pad_right += 1
        windows = list(range(-pad_left, len(seq), self.W_size))
        splitted_seq = []
        for c, i in enumerate(windows):
            beg = max(0, i)
            end = min(len(seq), i + self.W_size)
            label_ = labels
            if labels:
                label_ = [label if self._single_label[i] else label[beg:end] for i, label in enumerate(labels)]
            data_ = [dat if self._single_data[i] else dat[beg:end] for i, dat in enumerate(data)]
            if c == 0:
                splitted_seq.append(('{}_{}'.format(name, c), name, data_, label_, beg, end, pad_left))
            else:
                splitted_seq.append(('{}_{}'.format(name, c), name, data_, label_, beg, end, 0))
        return splitted_seq

    def __len__(self):
        return int(np.ceil(len(self.windowed_data) / self.batch_size))

    def __getitem__(self, idx):
        batch_data = self.windowed_data[self.windowed_data['batch'] == idx]
        X = [encoder.encode_batch(batch_data, i) for i, encoder in enumerate(self.data_encoders)]
        if self.label_encoders is not None:
            y = [encoder.encode_batch(batch_data, i) for i, encoder in enumerate(self.label_encoders)]
            return X, y
        return X



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

VERSION = '2.0'
with open('requirements.txt') as f:
      install_reqs = f.read().splitlines()

setup(name='deepcoil',
      version=VERSION,
      description='Fast and accurate prediction of coiled coil domains in protein sequences',
      long_description=long_description,
      long_description_content_type="text/markdown",
      author='Jan Ludwiczak',
      author_email='j.ludwiczak@cent.uw.edu.pl',
      url='https://github.com/labstructbioinf/deepcoil',
      packages=find_packages(),
      install_requires=install_reqs,
      python_requires=">=3.6.1",
      setup_requires=['pytest-runner'],
      tests_require=['pytest'],
      include_package_data=True,
      scripts=['bin/deepcoil'],
      classifiers=[
            "Programming Language :: Python :: 3",
            "License :: OSI Approved :: MIT License",
      ],
      )
